;; Problem 55 24 October 2003

;; If we take 47, reverse and add, 47 + 74 = 121, which is palindromic.

;; Not all numbers produce palindromes so quickly. For example,

;; 349 + 943 = 1292,
;; 1292 + 2921 = 4213
;; 4213 + 3124 = 7337

;; That is, 349 took three iterations to arrive at a palindrome.

;; Although no one has proved it yet, it is thought that some numbers,
;; like 196, never produce a palindrome. A number that never forms a
;; palindrome through the reverse and add process is called a Lychrel
;; number. Due to the theoretical nature of these numbers, and for the
;; purpose of this problem, we shall assume that a number is Lychrel
;; until proven otherwise. In addition you are given that for every
;; number below ten-thousand, it will either (i) become a palindrome
;; in less than fifty iterations, or, (ii) no one, with all the
;; computing power that exists, has managed so far to map it to a
;; palindrome. In fact, 10677 is the first number to be shown to
;; require over fifty iterations before producing a palindrome:
;; 4668731596684224866951378664 (53 iterations, 28-digits).

;; Surprisingly, there are palindromic numbers that are themselves
;; Lychrel numbers; the first example is 4994.

;; How many Lychrel numbers are there below ten-thousand?

;; NOTE: Wording was modified slightly on 24 April 2007 to emphasise
;; the theoretical nature of Lychrel numbers.


(defun num-is-pal (num)
  (let ((str (format nil "~d" num)))
    (equal str (reverse str))))

(assert (num-is-pal 121))
(assert (not (num-is-pal 122)))

(defun num-rev (num)
  (read-from-string (reverse (format nil "~d" num))))

(assert (= 122 (num-rev 221)))

(defun is-lychrel? (num &optional (iterations 50))
  (loop for attempt from 1 to iterations
	for next = (+ (num-rev num) num)
	then (+ (num-rev next) next)
	do
	(when (num-is-pal next)
	  (return-from is-lychrel? nil)))
  t)

(assert (is-lychrel? 196))
(assert (not (is-lychrel? 47)))
  

(defun count-lychrel-below (cnt &optional (iterations 50))
  (let ((res (list)))
    (loop for target from 1 to cnt
	  do
	  (when (is-lychrel? target iterations)
	    (format t "found: ~a (in ~d iterations)~&" target iterations)
	    (push target res)))
    (format t "total # under ~d = ~a~&"
	    cnt
	    (length res))))

(count-lychrel-below 10000)